import { assert } from 'chai'
import axios from 'axios'
import store from './store/index.spec.js'
import { createPinia, setActivePinia } from 'pinia'
import { useMLResultsStore, useAllowedMetaDataStore } from '@/stores'
import { FakeAxios, assertRejects } from './testhelpers.js'
import { workerVersionsCacheSample, workerVersionsSample } from './samples.js'
import {
  Filter,
  NameFilter,
  RotationAngleFilter,
  TypeFilter,
  MLClassFilter,
  ConfidenceFilter,
  WorkerVersionFilter,
  MetadataNameFilter,
  MetadataValueFilter,
  BooleanFilter,
  StringFilter,
  UUIDFilter,
  getNavigationFilters
} from '@/filterbar'

describe('filterbar.js', () => {
  let mock, mlResultsStore, allowedMetadataStore

  before(() => {
    setActivePinia(createPinia())
    mock = new FakeAxios(axios)
    mlResultsStore = useMLResultsStore()
    allowedMetadataStore = useAllowedMetaDataStore()
  })

  beforeEach(() => {
    store.state.corpora.corpora.corpusid = {
      id: 'corpusid',
      name: 'Test Corpus'
    }
  })

  afterEach(() => {
    store.reset()
    allowedMetadataStore.$reset()
    mlResultsStore.$reset()
    mock.reset()
  })

  after('Removing Axios mock', () => {
    mock.restore()
  })

  describe('Filter', () => {
    it('has valid default attributes', () => {
      const filter = new Filter('an_awesome_filter')

      assert.strictEqual(filter.name, 'an_awesome_filter')
      assert.strictEqual(filter.displayName, 'An awesome filter')
      assert.strictEqual(filter.placeholder, 'Filter value')
      assert.strictEqual(filter.operatorName, undefined)
      assert.strictEqual(filter.operatorPlaceholder, 'Select an operator')
      assert.deepStrictEqual(filter.operators, { eq: '=' })
    })

    it('allows overriding default attributes', () => {
      const filter = new Filter('an_awesome_filter', {
        displayName: 'well_hello',
        placeholder: 'blah',
        operatorName: 'imperator_name',
        operatorPlaceholder: 'Select an imperator',
        operators: {
          sqrt: '√',
          and: '∧'
        }
      })

      assert.strictEqual(filter.name, 'an_awesome_filter')
      assert.strictEqual(filter.displayName, 'well_hello')
      assert.strictEqual(filter.placeholder, 'blah')
      assert.strictEqual(filter.operatorName, 'imperator_name')
      assert.strictEqual(filter.operatorPlaceholder, 'Select an imperator')
      assert.deepStrictEqual(filter.operators, {
        sqrt: '√',
        and: '∧'
      })
    })

    it('has no autocomplete', () => {
      const filter = new Filter('an_awesome_filter')

      assert.deepStrictEqual(filter.autocomplete('', 'eq'), [])
      assert.deepStrictEqual(filter.autocomplete('some input', 'eq'), [])
      assert.deepStrictEqual(filter.autocomplete('', 'invalid_operator'), [])
    })

    it('sorts suggestions alphabetically by displayed value', () => {
      const filter = new Filter('an_awesome_filter')
      const suggestions = [
        [42, 'something'],
        ['something', '42'],
        ['anything', 'anything']
      ]

      assert.deepStrictEqual(filter.sort([]), [])
      assert.deepStrictEqual(filter.sort(suggestions), [
        ['something', '42'],
        ['anything', 'anything'],
        [42, 'something']
      ])
      // The original array should not be mutated
      assert.deepStrictEqual(suggestions, [
        [42, 'something'],
        ['something', '42'],
        ['anything', 'anything']
      ])
    })

    it('displays a value as a string', () => {
      const filter = new Filter('an_awesome_filter')

      assert.strictEqual(filter.display('blah'), 'blah')
      assert.strictEqual(filter.display(42), '42')
      assert.strictEqual(filter.display(true), 'true')
    })

    it('validates anything', () => {
      const filter = new Filter('an_awesome_filter')

      assert.strictEqual(filter.validate('blah'), 'blah')
      assert.strictEqual(filter.validate(42), 42)
      assert.strictEqual(filter.validate(true), true)
      assert.deepStrictEqual(filter.validate({ a: 'b' }), { a: 'b' })
    })

    it('serializes anything as a string', () => {
      const filter = new Filter('an_awesome_filter')

      assert.strictEqual(filter.serialize('blah'), 'blah')
      assert.strictEqual(filter.serialize(42), '42')
      assert.strictEqual(filter.serialize(true), 'true')
    })

    it('does nothing on a deserialize', () => {
      const filter = new Filter('an_awesome_filter')

      assert.strictEqual(filter.deserialize('blah'), 'blah')
      assert.strictEqual(filter.deserialize(42), 42)
      assert.strictEqual(filter.deserialize(true), true)
      assert.deepStrictEqual(filter.deserialize({ a: 'b' }), { a: 'b' })
    })
  })

  describe('NameFilter', () => {
    it('has correct attributes', () => {
      const filter = new NameFilter()

      assert.strictEqual(filter.name, 'name')
      assert.strictEqual(filter.displayName, 'Name')
      assert.strictEqual(filter.placeholder, 'Filter value')
      assert.strictEqual(filter.operatorName, undefined)
      assert.strictEqual(filter.operatorPlaceholder, 'Select an operator')
      assert.deepStrictEqual(filter.operators, { eq: 'contains' })
    })

    it('has no autocomplete', () => {
      const filter = new NameFilter()

      assert.deepStrictEqual(filter.autocomplete('', 'eq'), [])
      assert.deepStrictEqual(filter.autocomplete('some input', 'eq'), [])
      assert.deepStrictEqual(filter.autocomplete('', 'invalid_operator'), [])
    })

    it('sorts suggestions alphabetically by displayed value', () => {
      const filter = new NameFilter()
      const suggestions = [
        [42, 'something'],
        ['something', '42'],
        ['anything', 'anything']
      ]

      assert.deepStrictEqual(filter.sort([]), [])
      assert.deepStrictEqual(filter.sort(suggestions), [
        ['something', '42'],
        ['anything', 'anything'],
        [42, 'something']
      ])
      // The original array should not be mutated
      assert.deepStrictEqual(suggestions, [
        [42, 'something'],
        ['something', '42'],
        ['anything', 'anything']
      ])
    })

    it('displays a value as a string', () => {
      const filter = new NameFilter()

      assert.strictEqual(filter.display('blah'), 'blah')
      assert.strictEqual(filter.display(42), '42')
      assert.strictEqual(filter.display(true), 'true')
    })

    it('validates anything', () => {
      const filter = new NameFilter()

      assert.strictEqual(filter.validate('blah'), 'blah')
      assert.strictEqual(filter.validate(42), 42)
      assert.strictEqual(filter.validate(true), true)
      assert.deepStrictEqual(filter.validate({ a: 'b' }), { a: 'b' })
    })

    it('serializes anything as a string', () => {
      const filter = new NameFilter()

      assert.strictEqual(filter.serialize('blah'), 'blah')
      assert.strictEqual(filter.serialize(42), '42')
      assert.strictEqual(filter.serialize(true), 'true')
    })

    it('does nothing on a deserialize', () => {
      const filter = new NameFilter()

      assert.strictEqual(filter.deserialize('blah'), 'blah')
      assert.strictEqual(filter.deserialize(42), 42)
      assert.strictEqual(filter.deserialize(true), true)
      assert.deepStrictEqual(filter.deserialize({ a: 'b' }), { a: 'b' })
    })
  })

  describe('RotationAngleFilter', () => {
    it('has correct attributes', () => {
      const filter = new RotationAngleFilter()

      assert.strictEqual(filter.name, 'rotation_angle')
      assert.strictEqual(filter.displayName, 'Rotation angle')
      assert.strictEqual(filter.placeholder, 'Angle between 0 and 359 degrees')
      assert.strictEqual(filter.operatorName, undefined)
      assert.strictEqual(filter.operatorPlaceholder, 'Select an operator')
      assert.deepStrictEqual(filter.operators, { eq: '=' })
    })

    it('has no autocomplete', () => {
      const filter = new RotationAngleFilter()

      assert.deepStrictEqual(filter.autocomplete('', 'eq'), [])
      assert.deepStrictEqual(filter.autocomplete('some input', 'eq'), [])
      assert.deepStrictEqual(filter.autocomplete('', 'invalid_operator'), [])
    })

    it('sorts suggestions alphabetically by displayed value', () => {
      const filter = new RotationAngleFilter()
      const suggestions = [
        [42, 'something'],
        ['something', '42'],
        ['anything', 'anything']
      ]

      assert.deepStrictEqual(filter.sort([]), [])
      assert.deepStrictEqual(filter.sort(suggestions), [
        ['something', '42'],
        ['anything', 'anything'],
        [42, 'something']
      ])
      // The original array should not be mutated
      assert.deepStrictEqual(suggestions, [
        [42, 'something'],
        ['something', '42'],
        ['anything', 'anything']
      ])
    })

    it('displays a value as a string', () => {
      const filter = new RotationAngleFilter()

      assert.strictEqual(filter.display('blah'), 'blah')
      assert.strictEqual(filter.display(42), '42')
      assert.strictEqual(filter.display(true), 'true')
    })

    describe('validate', () => {
      it('accepts angles', () => {
        const filter = new RotationAngleFilter()

        assert.strictEqual(filter.validate('42'), 42)
        assert.strictEqual(filter.validate(42.0), 42)
        assert.strictEqual(filter.validate(180), 180)
        assert.strictEqual(filter.validate(0), 0)
        assert.strictEqual(filter.validate(359), 359)
      })

      it('requires a number', () => {
        const filter = new RotationAngleFilter()

        assert.throws(
          () => filter.validate('upright'),
          Error,
          'A rotation angle must be a number between 0 and 359 degrees'
        )
      })

      it('requires an integer', () => {
        const filter = new RotationAngleFilter()

        assert.throws(
          () => filter.validate('2.4'),
          Error,
          'A rotation angle must be a number between 0 and 359 degrees'
        )
      })

      it('requires a positive integer', () => {
        const filter = new RotationAngleFilter()

        assert.throws(
          () => filter.validate('-2'),
          Error,
          'A rotation angle must be a number between 0 and 359 degrees'
        )
      })

      it('requires an integer below 360', () => {
        const filter = new RotationAngleFilter()

        assert.throws(
          () => filter.validate('360'),
          Error,
          'A rotation angle must be a number between 0 and 359 degrees'
        )
      })
    })

    it('serializes anything as a string', () => {
      const filter = new RotationAngleFilter()

      assert.strictEqual(filter.serialize('blah'), 'blah')
      assert.strictEqual(filter.serialize(42), '42')
      assert.strictEqual(filter.serialize(true), 'true')
    })

    it('does nothing on a deserialize', () => {
      const filter = new RotationAngleFilter()

      assert.strictEqual(filter.deserialize('blah'), 'blah')
      assert.strictEqual(filter.deserialize(42), 42)
      assert.strictEqual(filter.deserialize(true), true)
      assert.deepStrictEqual(filter.deserialize({ a: 'b' }), { a: 'b' })
    })
  })

  describe('ConfidenceFilter', () => {
    it('has correct default attributes', () => {
      const filter = new ConfidenceFilter('classification_confidence')

      assert.strictEqual(filter.name, 'classification_confidence')
      assert.strictEqual(filter.displayName, 'Classification confidence')
      assert.strictEqual(filter.placeholder, 'Confidence between 0.0 and 1.0')
      assert.strictEqual(filter.operatorName, 'classification_confidence_operator')
      assert.strictEqual(filter.operatorPlaceholder, 'Select an operator')
      assert.deepStrictEqual(filter.operators, {
        eq: '=',
        gt: '>',
        lt: '<',
        gte: '>=',
        lte: '<='
      })
    })

    it('allows overriding the defaults', () => {
      const filter = new ConfidenceFilter('self_confidence', {
        displayName: 'lol',
        placeholder: 'You can do it!',
        operators: {
          a: 'ah',
          b: 'bee'
        },
        operatorName: 'meow',
        operatorPlaceholder: 'Shia LaBeouf believes in you'
      })

      assert.strictEqual(filter.name, 'self_confidence')
      assert.strictEqual(filter.displayName, 'lol')
      assert.strictEqual(filter.placeholder, 'You can do it!')
      assert.strictEqual(filter.operatorName, 'meow')
      assert.strictEqual(filter.operatorPlaceholder, 'Shia LaBeouf believes in you')
      assert.deepStrictEqual(filter.operators, {
        a: 'ah',
        b: 'bee'
      })
    })

    it('has no autocomplete', () => {
      const filter = new ConfidenceFilter('classification_confidence')

      assert.deepStrictEqual(filter.autocomplete('', 'eq'), [])
      assert.deepStrictEqual(filter.autocomplete('some input', 'eq'), [])
      assert.deepStrictEqual(filter.autocomplete('', 'invalid_operator'), [])
    })

    it('sorts suggestions alphabetically by displayed value', () => {
      const filter = new ConfidenceFilter('classification_confidence')
      const suggestions = [
        [42, 'something'],
        ['something', '42'],
        ['anything', 'anything']
      ]

      assert.deepStrictEqual(filter.sort([]), [])
      assert.deepStrictEqual(filter.sort(suggestions), [
        ['something', '42'],
        ['anything', 'anything'],
        [42, 'something']
      ])
      // The original array should not be mutated
      assert.deepStrictEqual(suggestions, [
        [42, 'something'],
        ['something', '42'],
        ['anything', 'anything']
      ])
    })

    it('displays a value as a string', () => {
      const filter = new ConfidenceFilter('classification_confidence')

      assert.strictEqual(filter.display('blah'), 'blah')
      assert.strictEqual(filter.display(42), '42')
      assert.strictEqual(filter.display(true), 'true')
    })

    describe('validate', () => {
      it('accepts confidences', () => {
        const filter = new ConfidenceFilter('classification_confidence')

        assert.strictEqual(filter.validate('0'), 0)
        assert.strictEqual(filter.validate(0.0), 0)
        assert.strictEqual(filter.validate('1'), 1)
        assert.strictEqual(filter.validate(1.0), 1)
        assert.strictEqual(filter.validate(0.1337), 0.1337)
      })

      it('requires a number', () => {
        const filter = new ConfidenceFilter('classification_confidence')

        assert.throws(
          () => filter.validate('unsure'),
          Error,
          'A confidence must be a number between 0.0 and 1.0'
        )
      })

      it('requires a positive number', () => {
        const filter = new ConfidenceFilter('classification_confidence')

        assert.throws(
          () => filter.validate('-.1'),
          Error,
          'A confidence must be a number between 0.0 and 1.0'
        )
      })

      it('requires a number below 1', () => {
        const filter = new ConfidenceFilter('classification_confidence')

        assert.throws(
          () => filter.validate('1.1'),
          Error,
          'A confidence must be a number between 0.0 and 1.0'
        )
      })

      it('requires a finite number', () => {
        const filter = new ConfidenceFilter('classification_confidence')

        assert.throws(
          () => filter.validate(Infinity),
          Error,
          'A confidence must be a number between 0.0 and 1.0'
        )
      })
    })

    it('serializes anything as a string', () => {
      const filter = new ConfidenceFilter('classification_confidence')

      assert.strictEqual(filter.serialize('blah'), 'blah')
      assert.strictEqual(filter.serialize(42), '42')
      assert.strictEqual(filter.serialize(true), 'true')
    })

    it('does nothing on a deserialize', () => {
      const filter = new ConfidenceFilter('classification_confidence')

      assert.strictEqual(filter.deserialize('blah'), 'blah')
      assert.strictEqual(filter.deserialize(42), 42)
      assert.strictEqual(filter.deserialize(true), true)
      assert.deepStrictEqual(filter.deserialize({ a: 'b' }), { a: 'b' })
    })
  })

  describe('TypeFilter', () => {
    beforeEach(() => {
      store.state.navigation.corpusId = 'corpusid'
      store.state.corpora.corpora.corpusid = {
        types: {
          page: {
            display_name: 'A page',
            slug: 'page',
            folder: false
          },
          text_line: {
            display_name: 'ligne de texte',
            slug: 'text_line',
            folder: false
          }
        }
      }
      store.state.corpora.corpora.othercorpus = {
        types: {
          unexpected: {
            display_name: 'spanish inquisition',
            slug: 'unexpected',
            folder: false
          }
        }
      }
    })

    it('has correct attributes', () => {
      const filter = new TypeFilter(store)

      assert.strictEqual(filter.name, 'type')
      assert.strictEqual(filter.displayName, 'Type')
      assert.strictEqual(filter.placeholder, 'Filter value')
      assert.strictEqual(filter.operatorName, undefined)
      assert.strictEqual(filter.operatorPlaceholder, 'Select an operator')
      assert.deepStrictEqual(filter.operators, { eq: '=' })
      assert.strictEqual(filter.store, store)
    })

    it("autocompletes with the current corpus' types", () => {
      const filter = new TypeFilter(store)

      assert.deepStrictEqual(filter.autocomplete(), [
        {
          display_name: 'A page',
          slug: 'page',
          folder: false
        },
        {
          display_name: 'ligne de texte',
          slug: 'text_line',
          folder: false
        }
      ])
    })

    it('sorts suggestions alphabetically by displayed value', () => {
      const filter = new TypeFilter(store)
      const suggestions = [
        [42, 'something'],
        ['something', '42'],
        ['anything', 'anything']
      ]

      assert.deepStrictEqual(filter.sort([]), [])
      assert.deepStrictEqual(filter.sort(suggestions), [
        ['something', '42'],
        ['anything', 'anything'],
        [42, 'something']
      ])
      // The original array should not be mutated
      assert.deepStrictEqual(suggestions, [
        [42, 'something'],
        ['something', '42'],
        ['anything', 'anything']
      ])
    })

    it("displays the type's display name", () => {
      const filter = new TypeFilter(store)

      assert.deepStrictEqual(
        filter.display({
          display_name: 'ligne de texte',
          slug: 'text_line',
          folder: false
        }),
        'ligne de texte'
      )
    })

    describe('validate', () => {
      it('finds a valid type from a display name', () => {
        const filter = new TypeFilter(store)

        assert.deepStrictEqual(filter.validate('a page'), {
          display_name: 'A page',
          slug: 'page',
          folder: false
        })
        assert.deepStrictEqual(filter.validate('LiGne De TexTE'), {
          display_name: 'ligne de texte',
          slug: 'text_line',
          folder: false
        })
      })

      it('throws on an invalid display name', () => {
        const filter = new TypeFilter(store)

        assert.throws(
          () => filter.validate('spanish inquisition'),
          Error,
          "Type 'spanish inquisition' not found."
        )
      })
    })

    it('serializes a type as a slug', () => {
      const filter = new TypeFilter(store)

      assert.deepStrictEqual(
        filter.serialize({
          display_name: 'ligne de texte',
          slug: 'text_line',
          folder: false
        }),
        'text_line'
      )
    })

    describe('deserialize', () => {
      it('finds a valid type from a slug', () => {
        const filter = new TypeFilter(store)

        assert.deepStrictEqual(filter.deserialize('page'), {
          display_name: 'A page',
          slug: 'page',
          folder: false
        })
        assert.deepStrictEqual(filter.deserialize('text_line'), {
          display_name: 'ligne de texte',
          slug: 'text_line',
          folder: false
        })
      })

      it('ignores invalid values', () => {
        const filter = new TypeFilter(store)

        assert.strictEqual(filter.deserialize('unexpected'), undefined)
        assert.strictEqual(filter.deserialize(null), undefined)
      })
    })
  })

  describe('MLClassFilter', () => {
    it('has correct attributes', () => {
      const filter = new MLClassFilter(store)

      assert.strictEqual(filter.name, 'class_id')
      assert.strictEqual(filter.displayName, 'Class')
      assert.strictEqual(filter.placeholder, 'Filter value')
      assert.strictEqual(filter.operatorName, undefined)
      assert.strictEqual(filter.operatorPlaceholder, 'Select an operator')
      assert.deepStrictEqual(filter.operators, { eq: '=' })
      assert.strictEqual(filter.store, store)
    })

    const classes = [
      {
        id: 'class1',
        name: 'Class one'
      },
      {
        id: 'class2',
        name: 'Class two'
      }
    ]

    describe('autocomplete', () => {
      it('autocompletes with classes', async () => {
        store.state.navigation.corpusId = 'corpusid'
        mock.onGet('/corpus/corpusid/classes/').reply(200, {
          count: 2,
          results: classes
        })

        const filter = new MLClassFilter(store)

        assert.deepStrictEqual(await filter.autocomplete(), classes)

        assert.deepStrictEqual(store.history, [
          {
            action: 'classification/listCorpusMLClasses',
            payload: {
              id: 'corpusid',
              search: null
            }
          },
          {
            mutation: 'classification/setHasMLClasses',
            payload: {
              id: 'corpusid',
              value: true
            }
          },
          {
            mutation: 'classification/setCount',
            payload: {
              id: 'corpusid',
              value: 2
            }
          },
          {
            mutation: 'classification/addMLClass',
            payload: {
              ...classes[0],
              corpus_id: 'corpusid'
            }
          },
          {
            mutation: 'classification/addMLClass',
            payload: {
              ...classes[1],
              corpus_id: 'corpusid'
            }
          }
        ])
      })

      it("handles the user's current input", async () => {
        store.state.navigation.corpusId = 'corpusid'
        mock.onGet('/corpus/corpusid/classes/', { params: { search: 'class' } }).reply(200, {
          count: 2,
          results: classes
        })

        const filter = new MLClassFilter(store)

        assert.deepStrictEqual(await filter.autocomplete('class'), classes)

        assert.deepStrictEqual(store.history, [
          {
            action: 'classification/listCorpusMLClasses',
            payload: {
              id: 'corpusid',
              search: 'class'
            }
          },
          {
            mutation: 'classification/addMLClass',
            payload: {
              ...classes[0],
              corpus_id: 'corpusid'
            }
          },
          {
            mutation: 'classification/addMLClass',
            payload: {
              ...classes[1],
              corpus_id: 'corpusid'
            }
          }
        ])
      })
    })

    it('sorts suggestions alphabetically by displayed value', () => {
      const filter = new MLClassFilter(store)
      const suggestions = [
        [42, 'something'],
        ['something', '42'],
        ['anything', 'anything']
      ]

      assert.deepStrictEqual(filter.sort([]), [])
      assert.deepStrictEqual(filter.sort(suggestions), [
        ['something', '42'],
        ['anything', 'anything'],
        [42, 'something']
      ])
      // The original array should not be mutated
      assert.deepStrictEqual(suggestions, [
        [42, 'something'],
        ['something', '42'],
        ['anything', 'anything']
      ])
    })

    it('display', () => {
      const filter = new MLClassFilter(store)

      assert.strictEqual(
        filter.display({
          id: 'class1',
          name: 'Class one'
        }),
        'Class one'
      )
    })

    describe('validate', () => {
      it('accepts an ML class', async () => {
        store.state.navigation.corpusId = 'corpusid'
        mock.onGet('/corpus/corpusid/classes/', { params: { search: 'class one' } }).reply(200, {
          count: 2,
          results: classes
        })

        const filter = new MLClassFilter(store)

        assert.deepStrictEqual(await filter.validate('class one'), {
          id: 'class1',
          name: 'Class one'
        })

        assert.deepStrictEqual(store.history, [
          {
            action: 'classification/listCorpusMLClasses',
            payload: {
              id: 'corpusid',
              search: 'class one'
            }
          },
          {
            mutation: 'classification/addMLClass',
            payload: {
              ...classes[0],
              corpus_id: 'corpusid'
            }
          },
          {
            mutation: 'classification/addMLClass',
            payload: {
              ...classes[1],
              corpus_id: 'corpusid'
            }
          }
        ])
      })

      it('throws on invalid values', async () => {
        store.state.navigation.corpusId = 'corpusid'
        mock.onGet('/corpus/corpusid/classes/', { params: { search: 'oh no' } }).reply(200, {
          count: 2,
          results: classes
        })

        const filter = new MLClassFilter(store)

        const err = await assertRejects(async () => await filter.validate('oh no'))
        assert.strictEqual(err.toString(), "Error: Unknown class 'oh no'")

        assert.deepStrictEqual(store.history, [
          {
            action: 'classification/listCorpusMLClasses',
            payload: {
              id: 'corpusid',
              search: 'oh no'
            }
          },
          {
            mutation: 'classification/addMLClass',
            payload: {
              ...classes[0],
              corpus_id: 'corpusid'
            }
          },
          {
            mutation: 'classification/addMLClass',
            payload: {
              ...classes[1],
              corpus_id: 'corpusid'
            }
          }
        ])
      })
    })

    it('serialize', () => {
      const filter = new MLClassFilter(store)

      assert.strictEqual(
        filter.serialize({
          id: 'class1',
          name: 'Class one'
        }),
        'class1'
      )
    })

    describe('deserialize', () => {
      it('handles an ML class UUID', async () => {
        store.state.navigation.corpusId = 'corpusid'
        mock.onGet('/corpus/corpusid/classes/class1/').reply(200, classes[0])

        const filter = new MLClassFilter(store)

        assert.deepStrictEqual(await filter.deserialize('class1'), {
          id: 'class1',
          name: 'Class one'
        })
      })

      it('handles invalid values', async () => {
        store.state.navigation.corpusId = 'corpusid'
        mock.onGet('/corpus/corpusid/classes/class3/').reply(404)

        const filter = new MLClassFilter(store)

        assert.strictEqual(await filter.deserialize('class3'), undefined)
      })
    })
  })

  describe('WorkerVersionFilter', () => {
    it('has correct attributes', () => {
      const filter = new WorkerVersionFilter(store)

      assert.strictEqual(filter.name, 'worker_version')
      assert.strictEqual(filter.displayName, 'Worker version')
      assert.strictEqual(filter.placeholder, 'Filter value')
      assert.strictEqual(filter.operatorName, undefined)
      assert.strictEqual(filter.operatorPlaceholder, 'Select an operator')
      assert.deepStrictEqual(filter.operators, { eq: '=' })
      assert.strictEqual(filter.store, store)
    })

    it('allows overriding the defaults', () => {
      const filter = new WorkerVersionFilter(store, 'transcription_worker_version', {
        displayName: 'A potato',
        placeholder: 'Your text here',
        operators: {
          a: 'ah',
          b: 'bee',
          o: 'oh'
        },
        operatorName: 'woof',
        operatorPlaceholder: 'Your blood type'
      })

      assert.strictEqual(filter.name, 'transcription_worker_version')
      assert.strictEqual(filter.displayName, 'A potato')
      assert.strictEqual(filter.placeholder, 'Your text here')
      assert.strictEqual(filter.operatorName, 'woof')
      assert.strictEqual(filter.operatorPlaceholder, 'Your blood type')
      assert.deepStrictEqual(filter.operators, {
        a: 'ah',
        b: 'bee',
        o: 'oh'
      })
    })

    describe('autocomplete', () => {
      it('autocompletes with worker versions or false', async () => {
        store.state.navigation.corpusId = 'corpusid'
        mlResultsStore.corpusWorkerVersionsCache.corpusid = Object.fromEntries(
          workerVersionsCacheSample.results.map(cache => [cache.id, cache])
        )

        const filter = new WorkerVersionFilter(store)

        assert.deepStrictEqual(await filter.autocomplete(), [
          workerVersionsSample.results[0],
          workerVersionsSample.results[1],
          false
        ])
      })

      it('fetches worker versions when needed', async () => {
        store.state.navigation.corpusId = 'corpusid'
        mlResultsStore.corpusWorkerVersionsCache = {}
        mock.onGet('/corpus/corpusid/versions/').reply(200, workerVersionsCacheSample)

        const filter = new WorkerVersionFilter(store)

        assert.deepStrictEqual(
          await filter.autocomplete(),
          [...workerVersionsCacheSample.results.map(cache => cache.worker_version), false]
        )
      })
    })

    it('sorts suggestions alphabetically by displayed value', () => {
      const filter = new WorkerVersionFilter(store)
      const suggestions = [
        [42, 'something'],
        ['something', '42'],
        ['anything', 'anything']
      ]

      assert.deepStrictEqual(filter.sort([]), [])
      assert.deepStrictEqual(filter.sort(suggestions), [
        ['something', '42'],
        ['anything', 'anything'],
        [42, 'something']
      ])
      // The original array should not be mutated
      assert.deepStrictEqual(suggestions, [
        [42, 'something'],
        ['something', '42'],
        ['anything', 'anything']
      ])
    })

    describe('display', () => {
      it('handles false', () => {
        const filter = new WorkerVersionFilter(store)

        assert.strictEqual(filter.display(false), 'No worker version')
      })

      it('handles a worker version with a revision', () => {
        const filter = new WorkerVersionFilter(store)

        assert.strictEqual(filter.display(workerVersionsSample.results[0]), 'Worker 1 (xxxxxxxx)')
        assert.strictEqual(filter.display(workerVersionsSample.results[1]), 'Worker 2 (yyyyyyyy)')
      })

      it('handles a worker version without a revision', () => {
        const filter = new WorkerVersionFilter(store)
        const version = { ...workerVersionsSample.results[0], revision: null, version: 42 }

        assert.strictEqual(filter.display(version), 'Worker 1 (version 42)')
      })
    })

    describe('validate', () => {
      it('handles false', async () => {
        const filter = new WorkerVersionFilter(store)

        assert.strictEqual(await filter.validate('false'), false)
        assert.strictEqual(await filter.validate(false), false)
      })

      it('handles a worker version', async () => {
        const filter = new WorkerVersionFilter(store)

        assert.deepStrictEqual(await filter.validate(workerVersionsSample.results[0]), workerVersionsSample.results[0])
      })

      it('handles a worker version ID', async () => {
        store.state.navigation.corpusId = 'corpusid'
        mlResultsStore.corpusWorkerVersionsCache.corpusid = Object.fromEntries(
          workerVersionsCacheSample.results.map(cache => [cache.id, cache])
        )

        const filter = new WorkerVersionFilter(store)

        assert.deepStrictEqual(await filter.validate('versionid1'), workerVersionsSample.results[0])
      })

      it('throws on invalid values', async () => {
        store.state.navigation.corpusId = 'corpusid'
        mlResultsStore.corpusWorkerVersionsCache.corpusid = Object.fromEntries(
          workerVersionsCacheSample.results.map(cache => [cache.id, cache])
        )

        const filter = new WorkerVersionFilter(store)

        const err = await assertRejects(async () => await filter.validate('the wrong version'))
        assert.strictEqual(err.toString(), "Error: Unknown worker version ID: 'the wrong version'")
      })

      it('fetches worker versions when needed', async () => {
        store.state.navigation.corpusId = 'corpusid'
        mlResultsStore.corpusWorkerVersionsCache = {}
        mock.onGet('/corpus/corpusid/versions/').reply(200, workerVersionsCacheSample)

        const filter = new WorkerVersionFilter(store)

        assert.deepStrictEqual(await filter.validate('versionid1'), workerVersionsSample.results[0])
      })
    })

    describe('serialize', () => {
      it('handles false', () => {
        const filter = new WorkerVersionFilter(store)

        assert.strictEqual(filter.serialize(false), false)
      })

      it('handles a worker version', () => {
        const filter = new WorkerVersionFilter(store)

        assert.strictEqual(filter.serialize(workerVersionsSample.results[0]), 'versionid1')
        assert.strictEqual(filter.serialize(workerVersionsSample.results[1]), 'versionid2')
      })
    })

    describe('deserialize', () => {
      it('handles false', async () => {
        const filter = new WorkerVersionFilter(store)

        assert.strictEqual(await filter.deserialize('false'), false)
        assert.strictEqual(await filter.deserialize(false), false)
      })

      it('handles a worker version', async () => {
        const filter = new WorkerVersionFilter(store)

        assert.deepStrictEqual(await filter.deserialize(workerVersionsSample.results[0]), workerVersionsSample.results[0])
      })

      it('handles a worker version ID', async () => {
        store.state.navigation.corpusId = 'corpusid'
        mlResultsStore.corpusWorkerVersionsCache.corpusid = Object.fromEntries(
          workerVersionsCacheSample.results.map(cache => [cache.id, cache])
        )

        const filter = new WorkerVersionFilter(store)

        assert.deepStrictEqual(await filter.deserialize('versionid1'), workerVersionsSample.results[0])
      })

      it('handles invalid values', async () => {
        store.state.navigation.corpusId = 'corpusid'
        mlResultsStore.corpusWorkerVersionsCache.corpusid = Object.fromEntries(
          workerVersionsCacheSample.results.map(cache => [cache.id, cache])
        )

        const filter = new WorkerVersionFilter(store)

        assert.strictEqual(await filter.deserialize('the wrong version'), undefined)
      })

      it('fetches worker versions when needed', async () => {
        store.state.navigation.corpusId = 'corpusid'
        mlResultsStore.corpusWorkerVersionsCache = {}
        mock.onGet('/corpus/corpusid/versions/').reply(200, workerVersionsCacheSample)

        const filter = new WorkerVersionFilter(store)

        assert.deepStrictEqual(await filter.deserialize('versionid2'), workerVersionsSample.results[1])
      })
    })
  })

  describe('MetadataNameFilter', () => {
    it('has correct attributes', () => {
      const filter = new MetadataNameFilter(store)

      assert.strictEqual(filter.name, 'metadata_name')
      assert.strictEqual(filter.displayName, 'Metadata name')
      assert.strictEqual(filter.placeholder, 'Filter value')
      assert.strictEqual(filter.operatorName, undefined)
      assert.strictEqual(filter.operatorPlaceholder, 'Select an operator')
      assert.deepStrictEqual(filter.operators, { eq: '=' })
      assert.strictEqual(filter.store, store)
    })

    const testAllowedMetadata = [
      {
        name: 'le url',
        type: 'url'
      },
      {
        name: 'le number',
        type: 'numeric'
      }
    ]

    describe('autocomplete', () => {
      beforeEach(() => {
        store.state.navigation.corpusId = 'corpusid'
      })

      it('autocompletes with allowed metadata', async () => {
        allowedMetadataStore.allowedMetadata.corpusid = testAllowedMetadata

        const filter = new MetadataNameFilter(store)

        assert.deepStrictEqual(await filter.autocomplete(), [
          'le url',
          'le number'
        ])
      })

      it('fetches metadata only when necessary', async () => {
        mock.onGet('/corpus/corpusid/allowed-metadata/').reply(200, {
          count: 2,
          number: 1,
          results: testAllowedMetadata
        })

        const filter = new MetadataNameFilter(store)

        assert.deepStrictEqual(await filter.autocomplete(), [
          'le url',
          'le number'
        ])

        assert.deepStrictEqual(allowedMetadataStore.allowedMetadata, {
          corpusid: testAllowedMetadata
        })
      })
    })

    it('allows any name', () => {
      allowedMetadataStore.allowedMetadata.corpusid = testAllowedMetadata

      const filter = new MetadataNameFilter(store)

      assert.deepStrictEqual(filter.validate('something'), 'something')
    })
  })

  describe('MetadataValueFilter', () => {
    it('has correct attributes', () => {
      const filter = new MetadataValueFilter()

      assert.strictEqual(filter.name, 'metadata_value')
      assert.strictEqual(filter.displayName, 'Metadata value')
      assert.strictEqual(filter.placeholder, 'Filter value')
      assert.strictEqual(filter.operatorName, 'metadata_operator')
      assert.strictEqual(filter.operatorPlaceholder, 'Select an operator')
      assert.deepStrictEqual(filter.operators, {
        eq: '=',
        gt: '>',
        lt: '<',
        gte: '>=',
        lte: '<=',
        contains: 'contains'
      })
    })

    describe('deserialize', () => {
      it('allows any string for equality and contains', () => {
        const filter = new MetadataValueFilter()

        assert.strictEqual(filter.deserialize('lol', 'eq'), 'lol')
        assert.strictEqual(filter.deserialize('lol', 'contains'), 'lol')
        // Numbers are still strings
        assert.strictEqual(filter.deserialize('42.4', 'eq'), '42.4')
        assert.strictEqual(filter.deserialize('42.4', 'contains'), '42.4')
      })

      it('returns a float for numeric operators', () => {
        const filter = new MetadataValueFilter()

        assert.strictEqual(filter.deserialize('42.4', 'gt'), 42.4)
      })

      it('ignores non-finite floats for numeric operators', () => {
        const filter = new MetadataValueFilter()

        assert.strictEqual(filter.deserialize('nope', 'gt'), undefined)
        assert.strictEqual(filter.deserialize('Infinity', 'gt'), undefined)
      })
    })

    describe('validate', () => {
      it('allows any string for equality and contains', () => {
        const filter = new MetadataValueFilter()

        assert.strictEqual(filter.validate('lol', 'eq'), 'lol')
        assert.strictEqual(filter.validate('lol', 'contains'), 'lol')
        // Numbers are still strings
        assert.strictEqual(filter.validate('42.4', 'eq'), '42.4')
        assert.strictEqual(filter.validate('42.4', 'contains'), '42.4')
      })

      it('returns a float for numeric operators', () => {
        const filter = new MetadataValueFilter()

        assert.strictEqual(filter.validate('42.4', 'gt'), 42.4)
      })

      it('throws an error on non-floats for numeric operators', () => {
        const filter = new MetadataValueFilter()

        const expectedError = 'Numeric comparison operators on metadata values require finite numbers'

        assert.throws(() => filter.validate('nope', 'gt'), Error, expectedError)
        assert.throws(() => filter.validate('Infinity', 'gt'), Error, expectedError)
      })
    })
  })

  describe('BooleanFilter', () => {
    it('has correct attributes', () => {
      const filter = new BooleanFilter('bowlean')

      assert.strictEqual(filter.name, 'bowlean')
      assert.strictEqual(filter.displayName, 'Bowlean')
      assert.strictEqual(filter.placeholder, 'Filter value')
      assert.strictEqual(filter.operatorName, undefined)
      assert.strictEqual(filter.operatorPlaceholder, 'Select an operator')
      assert.deepStrictEqual(filter.operators, { eq: '=' })
    })

    it('suggests both true and false', () => {
      const filter = new BooleanFilter('bowlean')

      assert.deepStrictEqual(filter.autocomplete('', 'eq'), [true, false])
    })

    it('preserves the order of suggestions', () => {
      const filter = new BooleanFilter('bowlean')

      assert.deepStrictEqual(filter.sort([]), [])
      assert.deepStrictEqual(filter.sort([true, false]), [true, false])
      assert.deepStrictEqual(filter.sort([false, true]), [false, true])
    })

    it('displays booleans', () => {
      const filter = new BooleanFilter('bowlean')

      assert.strictEqual(filter.display(true), 'Yes')
      assert.strictEqual(filter.display(false), 'No')
    })

    it('displays strings', () => {
      const filter = new BooleanFilter('bowlean')

      assert.strictEqual(filter.display('true'), 'Yes')
      assert.strictEqual(filter.display('false'), 'No')
    })

    describe('validate', () => {
      it('accepts booleans', () => {
        const filter = new BooleanFilter('bowlean')

        assert.strictEqual(filter.validate(true), true)
        assert.strictEqual(filter.validate(false), false)
      })

      it('accepts boolean strings', () => {
        const filter = new BooleanFilter('bowlean')

        for (const value of ['true', 'True', 'yes', 'YES']) assert.strictEqual(filter.validate(value), true)
        for (const value of ['false', 'False', 'no', 'NO']) assert.strictEqual(filter.validate(value), false)
      })

      it('throws on invalid values', () => {
        const filter = new BooleanFilter('bowlean')

        assert.throws(
          () => filter.validate('maybe'),
          Error,
          'Only Yes and No are allowed'
        )
      })
    })
  })

  describe('StringFilter', () => {
    it('has correct attributes', () => {
      const filter = new StringFilter('filter_name')

      assert.strictEqual(filter.name, 'filter_name')
      assert.strictEqual(filter.displayName, 'Filter name')
      assert.strictEqual(filter.placeholder, 'Filter value')
      assert.strictEqual(filter.operatorName, undefined)
      assert.strictEqual(filter.operatorPlaceholder, 'Select an operator')
      assert.deepStrictEqual(filter.operators, { eq: '=' })
    })

    it('has no autocomplete', () => {
      const filter = new StringFilter('filter_name')

      assert.deepStrictEqual(filter.autocomplete('', 'eq'), [])
      assert.deepStrictEqual(filter.autocomplete('some input', 'eq'), [])
      assert.deepStrictEqual(filter.autocomplete('', 'invalid_operator'), [])
    })

    it('displays a value as a string', () => {
      const filter = new StringFilter('filter_name')

      assert.strictEqual(filter.display('blah'), 'blah')
      assert.strictEqual(filter.display(42), '42')
      assert.strictEqual(filter.display(true), 'true')
    })

    it('validates anything', () => {
      const filter = new StringFilter('filter_name')

      assert.strictEqual(filter.validate('blah'), 'blah')
      assert.strictEqual(filter.validate(42), 42)
      assert.strictEqual(filter.validate(true), true)
      assert.deepStrictEqual(filter.validate({ a: 'b' }), { a: 'b' })
    })

    it('serializes anything as a string', () => {
      const filter = new StringFilter('filter_name')

      assert.strictEqual(filter.serialize('blah'), 'blah')
      assert.strictEqual(filter.serialize(42), '42')
      assert.strictEqual(filter.serialize(true), 'true')
    })

    it('does nothing on a deserialize', () => {
      const filter = new StringFilter('filter_name')

      assert.strictEqual(filter.deserialize('blah'), 'blah')
      assert.strictEqual(filter.deserialize(42), 42)
      assert.strictEqual(filter.deserialize(true), true)
      assert.deepStrictEqual(filter.deserialize({ a: 'b' }), { a: 'b' })
    })
  })

  describe('UUIDFilter', () => {
    it('has correct attributes', () => {
      const filter = new UUIDFilter('filter_name')

      assert.strictEqual(filter.name, 'filter_name')
      assert.strictEqual(filter.displayName, 'Filter name')
      assert.strictEqual(filter.placeholder, 'xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx')
      assert.strictEqual(filter.operatorName, undefined)
      assert.strictEqual(filter.operatorPlaceholder, 'Select an operator')
      assert.deepStrictEqual(filter.operators, { eq: '=' })
    })

    it('has no autocomplete', () => {
      const filter = new UUIDFilter('filter_name')

      assert.deepStrictEqual(filter.autocomplete('', 'eq'), [])
      assert.deepStrictEqual(filter.autocomplete('some input', 'eq'), [])
      assert.deepStrictEqual(filter.autocomplete('', 'invalid_operator'), [])
    })

    it('displays a value as a string', () => {
      const filter = new UUIDFilter('filter_name')

      assert.strictEqual(filter.display('blah'), 'blah')
      assert.strictEqual(filter.display(42), '42')
      assert.strictEqual(filter.display(true), 'true')
    })

    describe('validate', () => {
      it('accepts a UUID', () => {
        const filter = new UUIDFilter('filter_name')

        assert.strictEqual(filter.validate('aaaaaaaa-aaaa-aaaa-aaaa-aaaaaaaaaaaa'), 'aaaaaaaa-aaaa-aaaa-aaaa-aaaaaaaaaaaa')
        assert.strictEqual(filter.validate('12345678-90AB-CDEF-abcd-efAaAaAaAaAa'), '12345678-90ab-cdef-abcd-efaaaaaaaaaa')
      })

      it('rejects anything else', () => {
        const filter = new UUIDFilter('filter_name')

        for (const value of ['', 'a', '1', 'xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx', '00000000-0000-0000-0-000000000000']) {
          assert.throws(
            () => filter.validate(value),
            Error,
            'Invalid UUID'
          )
        }
      })
    })

    it('serializes anything as a string', () => {
      const filter = new UUIDFilter('filter_name')

      assert.strictEqual(filter.serialize('blah'), 'blah')
      assert.strictEqual(filter.serialize(42), '42')
      assert.strictEqual(filter.serialize(true), 'true')
    })

    it('does nothing on a deserialize', () => {
      const filter = new UUIDFilter('filter_name')

      assert.strictEqual(filter.deserialize('blah'), 'blah')
      assert.strictEqual(filter.deserialize(42), 42)
      assert.strictEqual(filter.deserialize(true), true)
      assert.deepStrictEqual(filter.deserialize({ a: 'b' }), { a: 'b' })
    })
  })

  describe('getNavigationFilters', () => {
    it('returns no filters without a selected corpus', () => {
      assert.deepStrictEqual(getNavigationFilters(store), [])
    })

    it('returns correct filters in a corpus', () => {
      store.state.navigation.corpusId = 'corpusid'
      assert.deepStrictEqual(getNavigationFilters(store), [
        new NameFilter(),
        new RotationAngleFilter(),
        new BooleanFilter('mirrored'),
        new WorkerVersionFilter(store),
        new MetadataNameFilter(store),
        new ConfidenceFilter('confidence'),
        new ConfidenceFilter('transcription_confidence'),
        new WorkerVersionFilter(store, 'transcription_worker_version'),
        new StringFilter('creator_email'),
        new UUIDFilter('worker_run', { displayName: 'Worker run ID' }),
        new UUIDFilter('transcription_worker_run', { displayName: 'Transcription worker run ID' }),
        new TypeFilter(store),
        new BooleanFilter('folder'),
        new MLClassFilter(store),
        new ConfidenceFilter('classification_confidence'),
        new BooleanFilter('classification_high_confidence', { displayName: 'High confidence classification' }),
        new BooleanFilter('top_level')
      ])
    })

    it('returns correct filters in a folder', () => {
      store.state.navigation.corpusId = 'corpusid'
      store.state.navigation.elementId = 'folderid'
      assert.deepStrictEqual(getNavigationFilters(store), [
        new NameFilter(),
        new RotationAngleFilter(),
        new BooleanFilter('mirrored'),
        new WorkerVersionFilter(store),
        new MetadataNameFilter(store),
        new ConfidenceFilter('confidence'),
        new ConfidenceFilter('transcription_confidence'),
        new WorkerVersionFilter(store, 'transcription_worker_version'),
        new StringFilter('creator_email'),
        new UUIDFilter('worker_run', { displayName: 'Worker run ID' }),
        new UUIDFilter('transcription_worker_run', { displayName: 'Transcription worker run ID' }),
        new TypeFilter(store),
        new BooleanFilter('folder'),
        new MLClassFilter(store),
        new ConfidenceFilter('classification_confidence'),
        new BooleanFilter('classification_high_confidence', { displayName: 'High confidence classification' }),
        new BooleanFilter('recursive')
      ])
    })

    it('does not return the type filter with a set folder filter', () => {
      store.state.navigation.corpusId = 'corpusid'
      assert.deepStrictEqual(getNavigationFilters(store, { folder: true }), [
        new NameFilter(),
        new RotationAngleFilter(),
        new BooleanFilter('mirrored'),
        new WorkerVersionFilter(store),
        new MetadataNameFilter(store),
        new ConfidenceFilter('confidence'),
        new ConfidenceFilter('transcription_confidence'),
        new WorkerVersionFilter(store, 'transcription_worker_version'),
        new StringFilter('creator_email'),
        new UUIDFilter('worker_run', { displayName: 'Worker run ID' }),
        new UUIDFilter('transcription_worker_run', { displayName: 'Transcription worker run ID' }),
        new BooleanFilter('folder'),
        new MLClassFilter(store),
        new ConfidenceFilter('classification_confidence'),
        new BooleanFilter('classification_high_confidence', { displayName: 'High confidence classification' }),
        new BooleanFilter('top_level')
      ])
    })

    it('does not return the folder filter with a set type filter', () => {
      store.state.navigation.corpusId = 'corpusid'
      assert.deepStrictEqual(getNavigationFilters(store, { type: 'folder' }), [
        new NameFilter(),
        new RotationAngleFilter(),
        new BooleanFilter('mirrored'),
        new WorkerVersionFilter(store),
        new MetadataNameFilter(store),
        new ConfidenceFilter('confidence'),
        new ConfidenceFilter('transcription_confidence'),
        new WorkerVersionFilter(store, 'transcription_worker_version'),
        new StringFilter('creator_email'),
        new UUIDFilter('worker_run', { displayName: 'Worker run ID' }),
        new UUIDFilter('transcription_worker_run', { displayName: 'Transcription worker run ID' }),
        new TypeFilter(store),
        new MLClassFilter(store),
        new ConfidenceFilter('classification_confidence'),
        new BooleanFilter('classification_high_confidence', { displayName: 'High confidence classification' }),
        new BooleanFilter('top_level')
      ])
    })

    it('adds a metadata value filter with a set metadata name filter', () => {
      store.state.navigation.corpusId = 'corpusid'
      assert.deepStrictEqual(getNavigationFilters(store, { metadata_name: 'something' }), [
        new NameFilter(),
        new RotationAngleFilter(),
        new BooleanFilter('mirrored'),
        new WorkerVersionFilter(store),
        new MetadataNameFilter(store),
        new ConfidenceFilter('confidence'),
        new ConfidenceFilter('transcription_confidence'),
        new WorkerVersionFilter(store, 'transcription_worker_version'),
        new StringFilter('creator_email'),
        new UUIDFilter('worker_run', { displayName: 'Worker run ID' }),
        new UUIDFilter('transcription_worker_run', { displayName: 'Transcription worker run ID' }),
        new MetadataValueFilter(),
        new TypeFilter(store),
        new BooleanFilter('folder'),
        new MLClassFilter(store),
        new ConfidenceFilter('classification_confidence'),
        new BooleanFilter('classification_high_confidence', { displayName: 'High confidence classification' }),
        new BooleanFilter('top_level')
      ])
    })

    it('excludes the ML class filter when there are no ML classes', () => {
      store.state.navigation.corpusId = 'corpusid'
      store.state.classification.hasMLClasses.corpusid = false
      assert.deepStrictEqual(getNavigationFilters(store), [
        new NameFilter(),
        new RotationAngleFilter(),
        new BooleanFilter('mirrored'),
        new WorkerVersionFilter(store),
        new MetadataNameFilter(store),
        new ConfidenceFilter('confidence'),
        new ConfidenceFilter('transcription_confidence'),
        new WorkerVersionFilter(store, 'transcription_worker_version'),
        new StringFilter('creator_email'),
        new UUIDFilter('worker_run', { displayName: 'Worker run ID' }),
        new UUIDFilter('transcription_worker_run', { displayName: 'Transcription worker run ID' }),
        new TypeFilter(store),
        new BooleanFilter('folder'),
        new BooleanFilter('top_level')
      ])
    })
  })
})
