import { assert } from 'chai'
import axios from 'axios'
import { useAllowedMetaDataStore, useNotificationStore } from '@/stores'
import { assertRejects, FakeAxios } from '../testhelpers'
import { createPinia, setActivePinia } from 'pinia'

describe('allowedMetadata', () => {
  describe('actions', () => {
    let mock, store, notificationStore

    before('Setting up mocks', () => {
      mock = new FakeAxios(axios)
      setActivePinia(createPinia())
      store = useAllowedMetaDataStore()
      notificationStore = useNotificationStore()
    })

    afterEach(() => {
      // Remove any handlers, but leave mocking in place
      mock.reset()
      store.$reset()
      notificationStore.$reset()
    })

    after('Removing Axios mock', () => {
      // Remove mocking entirely
      mock.restore()
    })

    describe('setCorpusAllowedMetadata', () => {
      it('updates allowed metadata on a corpus', () => {
        store.allowedMetadata = {
          corpus1: [
            { type: 'A', name: 'A' },
            { type: 'B', name: 'B' }
          ]
        }
        const addedAllowedMeta = [
          { type: 'A', name: 'A' },
          { type: 'A', name: 'B' },
          { type: 'B', name: 'A' }
        ]

        store.setCorpusAllowedMetadata('corpus1', addedAllowedMeta)

        assert.deepStrictEqual(store.allowedMetadata, {
          corpus1: [
            { type: 'A', name: 'A' },
            { type: 'B', name: 'B' },
            { type: 'A', name: 'B' },
            { type: 'B', name: 'A' }
          ]
        })
      })
    })

    describe('listAllowedMetadata', () => {
      it('recursively lists allowed metadata for a corpus', async () => {
        const pages = [{
          count: 3,
          number: 1,
          next: 'nexpage',
          results: [{ type: 'A', name: 'A' }, { type: 'A', name: 'B' }]
        }, {
          count: 3,
          number: 2,
          next: null,
          results: [{ type: 'B', name: 'B' }]
        }]
        mock.onGet('/corpus/corpusid/allowed-metadata/', { params: { page: 1 } }).reply(200, pages[0])
        mock.onGet('/corpus/corpusid/allowed-metadata/', { params: { page: 2 } }).reply(200, pages[1])

        await store.listAllowedMetadata('corpusid')

        assert.deepStrictEqual(store.allowedMetadata, {
          corpusid: [
            { name: 'A', type: 'A' },
            { name: 'B', type: 'A' },
            { name: 'B', type: 'B' }
          ]
        })
      })
    })

    describe('createAllowedMetadata', () => {
      it('creates a new allowed metadata in a corpus', async () => {
        store.allowedMetadata = {
          someCorpus: [
            { id: 'oneId', name: 'oneName', type: 'oneType' }
          ]
        }
        const response = {
          id: 'metaId',
          type: 'someType',
          name: 'someName'
        }
        mock.onPost('/corpus/someCorpus/allowed-metadata/').reply(201, response)

        await store.createAllowedMetadata('someCorpus', { type: 'someType', name: 'someName' })

        assert.deepStrictEqual(store.allowedMetadata, {
          someCorpus: [
            { id: 'oneId', name: 'oneName', type: 'oneType' },
            { id: 'metaId', name: 'someName', type: 'someType' }
          ]
        })
      })

      it('handles errors', async () => {
        mock.onPost('/corpus/someCorpus/allowed-metadata/').reply(400)
        await assertRejects(async () => store.createAllowedMetadata('someCorpus', { type: 'someType', name: 'someName' }))
      })
    })

    describe('deleteAllowedMetadata', () => {
      it('removes an allowed metadata (name, type) couple from a corpus', async () => {
        store.allowedMetadata = {
          someCorpus: [
            { id: 'oneId', name: 'oneName', type: 'oneType' },
            { id: 'twoId', name: 'twoName', type: 'twoType' },
            { id: 'threeId', name: 'threeName', type: 'threeType' }
          ]
        }
        mock.onDelete('/corpus/someCorpus/allowed-metadata/twoId/').reply(204)
        await store.deleteAllowedMetadata('someCorpus', 'twoId')
        assert.deepStrictEqual(store.allowedMetadata, {
          someCorpus: [
            { id: 'oneId', name: 'oneName', type: 'oneType' },
            { id: 'threeId', name: 'threeName', type: 'threeType' }
          ]
        })
      })
      it('handles errors', async () => {
        mock.onDelete('/corpus/someCorpus/allowed-metadata/missingId/').reply(400)
        await assertRejects(async () => await store.deleteAllowedMetadata('someCorpus', 'missingId'))
      })
    })

    describe('updateAllowedMetadata', () => {
      it('updates an existing allowed metadata (name, type) couple in a corpus', async () => {
        store.allowedMetadata = {
          someCorpus: [
            { id: 'oneId', name: 'oneName', type: 'oneType' },
            { id: 'twoId', name: 'twoName', type: 'twoType' },
            { id: 'threeId', name: 'threeName', type: 'threeType' }
          ]
        }
        const response = {
          id: 'threeId',
          type: 'fourType',
          name: 'threeName'
        }
        mock.onPatch('/corpus/someCorpus/allowed-metadata/threeId/').reply(200, response)
        await store.updateAllowedMetadata('someCorpus', { id: 'threeId', type: 'fourType', name: 'threeName' })
        assert.deepStrictEqual(store.allowedMetadata, {
          someCorpus: [
            { id: 'oneId', name: 'oneName', type: 'oneType' },
            { id: 'twoId', name: 'twoName', type: 'twoType' },
            { id: 'threeId', name: 'threeName', type: 'fourType' }
          ]
        })
      })
      it('handles errors', async () => {
        mock.onPatch('/corpus/someCorpus/allowed-metadata/missingId/').reply(400)
        await assertRejects(async () => await store.updateAllowedMetadata('someCorpus', { id: 'missingId', name: 'someName', type: 'someType' }))
      })
    })
  })
})
