import axios from 'axios'
import { assert } from 'chai'
import { createPinia, setActivePinia } from 'pinia'
import { useMetaDataStore } from '@/stores'
import { FakeAxios } from '../testhelpers'

describe('metadata', () => {
  describe('actions', () => {
    let mock, store

    before('Setting up mocks', () => {
      mock = new FakeAxios(axios)
      setActivePinia(createPinia())
      store = useMetaDataStore()
    })

    afterEach(() => {
      // Remove any handlers, but leave mocking in place
      mock.reset()
      store.$reset()
    })

    after('Removing Axios mock', () => {
      mock.restore()
    })

    describe('createMetadata', () => {
      it('create a metadata for an element', async () => {
        const payload = {
          type: 'text',
          name: 'Metadata',
          value: 'metadata value',
          entity_id: 'entity_1',
          worker_run_id: 'worker_run_1'
        }
        mock.onPost('/element/element_1/metadata/').reply(201, {
          id: 'metadata_1',
          type: 'text',
          name: 'Metadata',
          value: 'metadata value',
          entity_id: 'entity_1',
          worker_run_id: 'worker_run_1'
        })

        await store.createMetadata('element_1', payload)

        assert.deepStrictEqual(store.metadata, {
          element_1: [
            {
              id: 'metadata_1',
              type: 'text',
              name: 'Metadata',
              value: 'metadata value',
              entity_id: 'entity_1',
              worker_run_id: 'worker_run_1'
            }
          ]
        })
      })
    })

    describe('updateMetadata', () => {
      it('Update metadata of an element', async () => {
        const payload = {
          id: 'metadata_1',
          type: 'numeric',
          name: 'Metadata',
          value: 12,
          entity_id: 'entity_2'
        }

        store.metadata = {
          element_1: [
            {
              id: 'metadata_1',
              type: 'text',
              name: 'Metadata name',
              value: 'Metadata value',
              entity_id: 'entity_1'
            }
          ]
        }
        mock.onPatch('/metadata/metadata_1/').reply(200, payload)

        await store.updateMetadata('element_1', payload)

        assert.deepStrictEqual(store.metadata, {
          element_1: [
            {
              id: 'metadata_1',
              type: 'numeric',
              name: 'Metadata',
              value: 12,
              entity_id: 'entity_2'
            }
          ]
        })
      })
    })

    describe('deleteMetadata', () => {
      it('Delete a metadata', async () => {
        store.metadata = {
          element_1: [
            {
              id: 'metadata_1',
              type: 'text',
              name: 'Metadata',
              value: 'metadata value',
              entity_id: 'entity_1'
            }
          ]
        }

        mock.onDelete('/metadata/metadata_1/').reply(204)

        await store.deleteMetadata('element_1', {
          id: 'metadata_1'
        })

        assert.deepStrictEqual(store.metadata, { element_1: [] })
      })
    })

    describe('listMetadata', () => {
      it('List MetaData of an element', async () => {
        mock.onGet('/element/element_1/metadata/').reply(200, [
          {
            id: 'metadata_1',
            name: 'Metadata',
            type: 'text',
            value: 'metadata value',
            entity_id: 'entity_1'
          },
          {
            id: 'metadata_2',
            name: 'Metadata',
            type: 'text',
            value: 'metadata value',
            entity_id: 'entity_2'
          }
        ])

        await store.listMetadata('element_1')

        assert.deepStrictEqual(store.metadata, {
          element_1: [
            {
              id: 'metadata_1',
              name: 'Metadata',
              type: 'text',
              value: 'metadata value',
              entity_id: 'entity_1'
            },
            {
              id: 'metadata_2',
              name: 'Metadata',
              type: 'text',
              value: 'metadata value',
              entity_id: 'entity_2'
            }
          ]
        })
      })
    })
  })
})
